#include <atari.h>
#include <stdio.h>
#include <string.h>
#include <peekpoke.h>
#include <joystick.h>
#include "atari_defs.h"

#define SCREEN_SIZE 40

// 29 is '=' character in ANTIC character set
#define BRICK  29

// Global variables

unsigned char *video_ptr;
unsigned char total_bricks;
unsigned char paddle_pos=120;
unsigned char ball_x=120,ball_y=100;
signed char ball_dy=1;

// Joystick

extern char joy_driver;
unsigned char joy;

void set_colors()
{
	POKE(COLOR1,0xFF); // font color
	POKE(COLOR2,0);    // background color
}

void draw_bricks()
{
  for (total_bricks=0;total_bricks<4*SCREEN_SIZE;total_bricks++)
    video_ptr[total_bricks]=BRICK;
}

#define pmg_memory 0x2000
#define pmg_memory_ptr ((unsigned char *) 0x2000)

char paddle_gfx[]={ 0x3C, 0x7E, 0xFF, 0xFF };
char ball_gfx[]={ 0, 0x2, 0x2, 0x5, 0x5, 0x2, 0x2, 0 };

void place_ball()
{
	// +0x500 for the player 1,
	memcpy(pmg_memory_ptr+0x500+ball_y,ball_gfx,sizeof(ball_gfx));
	POKE(HPOSP1,ball_x); // horizontal position
}

void init_pmg()
{
	POKE(SDMCTL,DMACTL_ENABLE_PLAYER_DMA|DMACTL_ENABLE_MISSLE_DMA|DMACTL_NORMAL_PLAYFIELD|DMACTL_SINGLE_LINE_RESOLUTION|DMACTL_DMA_FETCH_INSTRUCTION); 
	// PMBASE points to our PMG memory page (page has 256 bytes) 
	POKE(PMBASE,pmg_memory/256);

	// Zero pmg memory
	memset(pmg_memory_ptr,0,2048);
	
	// turn on PMG for players and missiles
	POKE(GRACTL,PMG_PLAYERS | PMG_MISSILES); 

	// set PMG priority
	POKE(GPRIOR,1); 

	// init paddle 
	POKE(PCOLR0,0xFF);
	POKE(SIZEP0,SIZEP_DOUBLE); 
	POKE(HPOSP0,paddle_pos); // paddle position

	// +0x400 for the player 0, +192 for vertical position on the screen
	memcpy(pmg_memory_ptr+0x400+192,paddle_gfx,sizeof(paddle_gfx));

	// init ball
	POKE(PCOLR1,0x5F);
	POKE(SIZEP1,SIZEP_SINGLE); 
	place_ball();
}

void move_paddle_left()
{
	if (paddle_pos>48)
		paddle_pos-=2;
	POKE(HPOSP0,paddle_pos); 
}

void move_paddle_right()
{
	if (paddle_pos<190)
		paddle_pos+=2;
	POKE(HPOSP0,paddle_pos); 
}

void move_ball()
{
	ball_y+=ball_dy;
	if (ball_y==184 || ball_y==32)
		ball_dy*=-1;
	place_ball();
}

int main(void)
{
  joy_install(&joy_driver);
  _graphics(0);
  set_colors();
  init_pmg();

  // get screen memory pointer
  video_ptr=(unsigned char*)(PEEKW( PEEKW(560)+4 ));
  draw_bricks();
	
  for(;;)
  {
    // wait a moment and read the joystick state after that 
	POKE(CDTMV3,1);
	while(PEEK(CDTMV3));
    POKE(0x4D,0); // Disable Attract mode
    
    joy=joy_read(JOY_1);
    if (JOY_BTN_LEFT(joy))
		move_paddle_left();
    if (JOY_BTN_RIGHT(joy))
		move_paddle_right();

	move_ball();
  }
  return 0;
}
